@@ -1,21 +1,32 @@
 import datetime
 import threading
+import queue
+import uuid
+import requests
+
 import pika
 import logging
 import json
 import time
 import base64
-import uuid
 import os
 
-from typing import Callable, Dict, List, Optional, Union
+from typing import Callable, Dict, Optional, Union
+from sseclient import SSEClient
 from pika.exceptions import UnroutableError, NackError
 from pycti.api.opencti_api_client import OpenCTIApiClient
 from pycti.connector.opencti_connector import OpenCTIConnector
+from pycti.utils.opencti_stix2_splitter import OpenCTIStix2Splitter
+
+EVENTS_QUEUE = queue.Queue()
 
 
 def get_config_variable(
-    env_var: str, yaml_path: list, config: Dict = {}, isNumber: Optional[bool] = False
+    env_var: str,
+    yaml_path: list,
+    config: Dict = {},
+    isNumber: Optional[bool] = False,
+    default=None,
 ) -> Union[bool, int, None, str]:
     """[summary]
 
@@ -31,9 +42,9 @@ def get_config_variable(
         if yaml_path[0] in config and yaml_path[1] in config[yaml_path[0]]:
             result = config[yaml_path[0]][yaml_path[1]]
         else:
-            return None
+            return default
     else:
-        return None
+        return default
 
     if result == "yes" or result == "true" or result == "True":
         return True
@@ -92,17 +103,25 @@ class ListenQueue(threading.Thread):
         channel.basic_ack(delivery_tag=method.delivery_tag)
 
     def _data_handler(self, json_data):
-        job_id = json_data["job_id"] if "job_id" in json_data else None
+        # Set the API headers
+        work_id = json_data["internal"]["work_id"]
+        applicant_id = json_data["internal"]["applicant_id"]
+        self.helper.work_id = work_id
+        if applicant_id is not None:
+            self.helper.applicant_id = applicant_id
+            self.helper.api.set_applicant_id_header(applicant_id)
+        # Execute the callback
         try:
-            work_id = json_data["work_id"]
-            self.helper.current_work_id = work_id
-            self.helper.api.job.update_job(job_id, "progress", ["Starting process"])
-            messages = self.callback(json_data)
-            self.helper.api.job.update_job(job_id, "complete", messages)
+            self.helper.api.work.to_received(
+                work_id, "Connector ready to process the operation"
+            )
+            message = self.callback(json_data["event"])
+            self.helper.api.work.to_processed(work_id, message)
+
         except Exception as e:
             logging.exception("Error in message processing, reporting error to API")
             try:
-                self.helper.api.job.update_job(job_id, "error", [str(e)])
+                self.helper.api.work.to_processed(work_id, str(e), True)
             except:
                 logging.error("Failing reporting the processing")
 
@@ -165,6 +184,67 @@ class PingAlive(threading.Thread):
         self.ping()
 
 
+class StreamCatcher(threading.Thread):
+    def __init__(
+        self,
+        opencti_url,
+        opencti_token,
+        connector_last_event_id,
+        last_event_id,
+        stream_connection_id,
+    ):
+        threading.Thread.__init__(self)
+        self.opencti_url = opencti_url
+        self.opencti_token = opencti_token
+        self.connector_last_event_id = connector_last_event_id
+        self.last_event_id = last_event_id
+        self.stream_connection_id = stream_connection_id
+
+    def get_range(self, from_id):
+        payload = {
+            "from": from_id,
+            "size": 2000,
+            "connectionId": self.stream_connection_id,
+        }
+        headers = {"Authorization": "Bearer " + self.opencti_token}
+        r = requests.post(
+            self.opencti_url + "/stream/history", json=payload, headers=headers
+        )
+        return r.json()["lastEventId"]
+
+    def run(self):
+        from_event_id = self.connector_last_event_id
+        from_event_timestamp = 0
+        last_event_timestamp = int(self.last_event_id.split("-")[0])
+        while (
+            from_event_timestamp <= last_event_timestamp
+            and from_event_id != self.last_event_id
+        ):
+            from_event_id = self.get_range(from_event_id)
+            from_event_timestamp = int(from_event_id.split("-")[0])
+        logging.info("Events catchup requests done.")
+
+
+class StreamProcessor(threading.Thread):
+    def __init__(self, message_callback, get_state, set_state):
+        threading.Thread.__init__(self)
+        self.message_callback = message_callback
+        self.get_state = get_state
+        self.set_state = set_state
+
+    def run(self):
+        logging.info("All old events processed, consuming is now LIVE!")
+        while True:
+            msg = EVENTS_QUEUE.get(block=True, timeout=None)
+            self.message_callback(msg)
+            state = self.get_state()
+            if state is not None:
+                state["connectorLastEventId"] = msg.id
+                self.set_state(state)
+            else:
+                self.set_state({"connectorLastEventId": msg.id})
+
+
 class OpenCTIConnectorHelper:
     """Python API for OpenCTI connector
 
@@ -199,6 +279,9 @@ class OpenCTIConnectorHelper:
         self.connect_scope = get_config_variable(
             "CONNECTOR_SCOPE", ["connector", "scope"], config
         )
+        self.connect_auto = get_config_variable(
+            "CONNECTOR_AUTO", ["connector", "auto"], config, False, False
+        )
         self.log_level = get_config_variable(
             "CONNECTOR_LOG_LEVEL", ["connector", "log_level"], config
         )
@@ -213,14 +296,18 @@ class OpenCTIConnectorHelper:
         self.api = OpenCTIApiClient(
             self.opencti_url, self.opencti_token, self.log_level
         )
-        self.current_work_id = None
-
         # Register the connector in OpenCTI
         self.connector = OpenCTIConnector(
-            self.connect_id, self.connect_name, self.connect_type, self.connect_scope
+            self.connect_id,
+            self.connect_name,
+            self.connect_type,
+            self.connect_scope,
+            self.connect_auto,
         )
         connector_configuration = self.api.connector.register(self.connector)
         self.connector_id = connector_configuration["id"]
+        self.work_id = None
+        self.applicant_id = connector_configuration["connector_user"]["id"]
         self.connector_state = connector_configuration["connector_state"]
         self.config = connector_configuration["config"]
 
@@ -230,9 +317,8 @@ class OpenCTIConnectorHelper:
         )
         self.ping.start()
 
-        # Initialize caching
-        self.cache_index = {}
-        self.cache_added = []
+    def get_name(self):
+        return self.connect_name
 
     def set_state(self, state) -> None:
         """sets the connector state
@@ -259,7 +345,7 @@ class OpenCTIConnectorHelper:
         except:
             return None
 
-    def listen(self, message_callback: Callable[[Dict], List[str]]) -> None:
+    def listen(self, message_callback: Callable[[str, Dict], str]) -> None:
         """listen for messages and register callback function
 
         :param message_callback: callback function to process messages
@@ -269,6 +355,92 @@ class OpenCTIConnectorHelper:
         listen_queue = ListenQueue(self, self.config, message_callback)
         listen_queue.start()
 
+    def listen_stream(self, message_callback, url=None, token=None) -> None:
+        """listen for messages and register callback function
+
+        :param message_callback: callback function to process messages
+        """
+        current_state = self.get_state()
+        if current_state is None:
+            current_state = {"connectorLastEventId": "-"}
+
+        # Get the last event ID with the "connected" event msg
+        if url is not None and token is not None:
+            messages = SSEClient(
+                url + "/stream",
+                headers={"Authorization": "Bearer " + token},
+            )
+        else:
+            messages = SSEClient(
+                self.opencti_url + "/stream",
+                headers={"Authorization": "Bearer " + self.opencti_token},
+            )
+
+        # Create processor thread
+        processor_thread = StreamProcessor(
+            message_callback, self.get_state, self.set_state
+        )
+
+        last_event_id = None
+        for msg in messages:
+            try:
+                data = json.loads(msg.data)
+            except:
+                logging.error("Failed to load JSON: " + msg.data)
+                continue
+            if msg.event == "heartbeat":
+                logging.info("HEARTBEAT:" + str(msg))
+                continue
+            elif msg.event == "connected":
+                last_event_id = data["lastEventId"]
+                stream_connection_id = data["connectionId"]
+                # Launch processor if up to date
+                if current_state["connectorLastEventId"] == last_event_id:
+                    processor_thread.start()
+                # Launch catcher if not up to date
+                if last_event_id != current_state["connectorLastEventId"]:
+                    logging.info(
+                        "Some events have not been processed, catching them..."
+                    )
+                    if url is not None and token is not None:
+                        catcher_thread = StreamCatcher(
+                            url,
+                            token,
+                            current_state["connectorLastEventId"],
+                            last_event_id,
+                            stream_connection_id,
+                        )
+                    else:
+                        catcher_thread = StreamCatcher(
+                            self.opencti_url,
+                            self.opencti_token,
+                            current_state["connectorLastEventId"],
+                            last_event_id,
+                            stream_connection_id,
+                        )
+                    catcher_thread.start()
+            else:
+                # If receiving the last message, launch processor
+                if msg.id == last_event_id:
+                    message_callback(msg)
+                    processor_thread.start()
+                elif "catchup" not in data:
+                    EVENTS_QUEUE.put(msg)
+                else:
+                    message_callback(msg)
+                    state = self.get_state()
+                    if state is not None:
+                        state["connectorLastEventId"] = msg.id
+                        self.set_state(state)
+                    else:
+                        self.set_state({"connectorLastEventId": msg.id})
+
+    def get_opencti_url(self):
+        return self.opencti_url
+
+    def get_opencti_token(self):
+        return self.opencti_token
+
     def get_connector(self):
         return self.connector
 
@@ -280,7 +452,6 @@ class OpenCTIConnectorHelper:
 
     def date_now(self) -> str:
         """get the current date (UTC)
-
         :return: current datetime for utc
         :rtype: str
         """
@@ -291,11 +462,10 @@ class OpenCTIConnectorHelper:
         )
 
     # Push Stix2 helper
-    def send_stix2_bundle(
-        self, bundle, entities_types=None, update=False, split=True
-    ) -> list:
+    def send_stix2_bundle(self, bundle, **kwargs) -> list:
         """send a stix2 bundle to the API
 
+        :param work_id: a valid work id
         :param bundle: valid stix2 bundle
         :type bundle:
         :param entities_types: list of entities, defaults to None
@@ -308,19 +478,33 @@ class OpenCTIConnectorHelper:
         :return: list of bundles
         :rtype: list
         """
+        work_id = kwargs.get("work_id", self.work_id)
+        entities_types = kwargs.get("entities_types", None)
+        update = kwargs.get("update", False)
+        split = kwargs.get("split", True)
 
         if entities_types is None:
             entities_types = []
         if split:
-            bundles = self.split_stix2_bundle(bundle)
+            stix2_splitter = OpenCTIStix2Splitter()
+            bundles = stix2_splitter.split_bundle(bundle)
             if len(bundles) == 0:
                 raise ValueError("Nothing to import")
+            if work_id is not None:
+                self.api.work.add_expectations(work_id, len(bundles))
             pika_connection = pika.BlockingConnection(
                 pika.URLParameters(self.config["uri"])
             )
             channel = pika_connection.channel()
-            for bundle in bundles:
-                self._send_bundle(channel, bundle, entities_types, update)
+            for sequence, bundle in enumerate(bundles, start=1):
+                self._send_bundle(
+                    channel,
+                    bundle,
+                    work_id=work_id,
+                    entities_types=entities_types,
+                    sequence=sequence,
+                    update=update,
+                )
             channel.close()
             return bundles
         else:
@@ -328,11 +512,13 @@ class OpenCTIConnectorHelper:
                 pika.URLParameters(self.config["uri"])
             )
             channel = pika_connection.channel()
-            self._send_bundle(channel, bundle, entities_types, update)
+            self._send_bundle(
+                channel, bundle, entities_types=entities_types, update=update
+            )
             channel.close()
             return [bundle]
 
-    def _send_bundle(self, channel, bundle, entities_types=None, update=False) -> None:
+    def _send_bundle(self, channel, bundle, **kwargs) -> None:
         """send a STIX2 bundle to RabbitMQ to be consumed by workers
 
         :param channel: RabbitMQ channel
@@ -344,16 +530,14 @@ class OpenCTIConnectorHelper:
         :param update: whether to update data in the database, defaults to False
         :type update: bool, optional
         """
+        work_id = kwargs.get("work_id", None)
+        sequence = kwargs.get("sequence", 0)
+        update = kwargs.get("update", False)
+        entities_types = kwargs.get("entities_types", None)
 
         if entities_types is None:
             entities_types = []
 
-        # Create a job log expectation
-        if self.current_work_id is not None:
-            job_id = self.api.job.initiate_job(self.current_work_id)
-        else:
-            job_id = None
-
         # Validate the STIX 2 bundle
         # validation = validate_string(bundle)
         # if not validation.is_valid:
@@ -363,23 +547,30 @@ class OpenCTIConnectorHelper:
         # if self.current_work_id is None:
         #    raise ValueError('The job id must be specified')
         message = {
-            "job_id": job_id,
+            "applicant_id": self.applicant_id,
+            "action_sequence": sequence,
             "entities_types": entities_types,
-            "update": update,
-            "token": self.opencti_token,
             "content": base64.b64encode(bundle.encode("utf-8")).decode("utf-8"),
+            "update": update,
         }
+        if work_id is not None:
+            message["work_id"] = work_id
 
         # Send the message
         try:
             routing_key = "push_routing_" + self.connector_id
             channel.basic_publish(
-                self.config["push_exchange"], routing_key, json.dumps(message)
+                exchange=self.config["push_exchange"],
+                routing_key=routing_key,
+                body=json.dumps(message),
+                properties=pika.BasicProperties(
+                    delivery_mode=2,  # make message persistent
+                ),
             )
             logging.info("Bundle has been sent")
         except (UnroutableError, NackError) as e:
-            logging.error(f"Unable to send bundle, retry...{e}")
-            self._send_bundle(bundle, entities_types)
+            logging.error("Unable to send bundle, retry...", e)
+            self._send_bundle(channel, bundle, **kwargs)
 
     def split_stix2_bundle(self, bundle) -> list:
         """splits a valid stix2 bundle into a list of bundles
@@ -444,7 +635,7 @@ class OpenCTIConnectorHelper:
 
         :param item: valid stix2 item
         :type item:
-        :return: returns a dict of created_by_ref of object_marking_refs
+        :return: returns a dict of created_by of object_marking_refs
         :rtype: dict
         """
         # Marking definitions
@@ -505,8 +696,8 @@ class OpenCTIConnectorHelper:
         # Get embedded objects
         embedded_objects = self.stix2_get_embedded_objects(relationship)
         # Add created by ref
-        if embedded_objects["created_by_ref"] is not None:
-            items.append(embedded_objects["created_by_ref"])
+        if embedded_objects["created_by"] is not None:
+            items.append(embedded_objects["created_by"])
         # Add marking definitions
         if len(embedded_objects["object_marking_refs"]) > 0:
             items = items + embedded_objects["object_marking_refs"]
