@@ -1,8 +1,7 @@
 # coding: utf-8
 
 import json
-from pycti.utils.constants import CustomProperties, IdentityTypes
-from pycti.utils.opencti_stix2 import SPEC_VERSION
+from pycti.utils.constants import IdentityTypes
 
 
 class Identity:
@@ -10,77 +9,75 @@ class Identity:
         self.opencti = opencti
         self.properties = """
             id
-            stix_id_key
-            stix_label
+            standard_id
             entity_type
             parent_types
-            name
-            alias
-            description
-            contact_information
-            created
-            modified            
+            spec_version
             created_at
             updated_at
-            ... on Organization {
-                organization_class
-            }
-            createdByRef {
-                node {
+            createdBy {
+                ... on Identity {
                     id
+                    standard_id
                     entity_type
-                    stix_id_key
-                    stix_label
+                    parent_types
+                    spec_version
                     name
-                    alias
                     description
+                    roles
+                    contact_information
+                    x_opencti_aliases
                     created
                     modified
-                    ... on Organization {
-                        organization_class
-                    }
+                    objectLabel {
+                        edges {
+                            node {
+                                id
+                                value
+                                color
+                            }
+                        }
+                    }                    
                 }
-                relation {
-                    id
+                ... on Organization {
+                    x_opencti_organization_type
+                    x_opencti_reliability
                 }
-            }            
-            markingDefinitions {
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectMarking {
                 edges {
                     node {
                         id
+                        standard_id
                         entity_type
-                        stix_id_key
                         definition_type
                         definition
-                        level
-                        color
                         created
                         modified
-                    }
-                    relation {
-                        id
+                        x_opencti_order
+                        x_opencti_color
                     }
                 }
             }
-            tags {
+            objectLabel {
                 edges {
                     node {
                         id
-                        tag_type
                         value
                         color
                     }
-                    relation {
-                        id
-                    }
                 }
             }
             externalReferences {
                 edges {
                     node {
                         id
+                        standard_id
                         entity_type
-                        stix_id_key
                         source_name
                         description
                         url
@@ -89,11 +86,25 @@ class Identity:
                         created
                         modified
                     }
-                    relation {
-                        id
-                    }
                 }
             }
+            revoked
+            confidence
+            created
+            modified
+            identity_class
+            name
+            description
+            x_opencti_aliases
+            contact_information
+            ... on Individual {
+                x_opencti_firstname
+                x_opencti_lastname
+            }
+            ... on Organization {
+                x_opencti_organization_type
+                x_opencti_reliability
+            }
         """
 
     """
@@ -212,55 +223,85 @@ class Identity:
         :return Identity object
     """
 
-    def create_raw(self, **kwargs):
+    def create(self, **kwargs):
         type = kwargs.get("type", None)
-        name = kwargs.get("name", None)
-        description = kwargs.get("description", None)
-        contact_information = kwargs.get("contact_information", None)
-        alias = kwargs.get("alias", None)
-        id = kwargs.get("id", None)
-        stix_id_key = kwargs.get("stix_id_key", None)
+        stix_id = kwargs.get("stix_id", None)
+        created_by = kwargs.get("createdBy", None)
+        object_marking = kwargs.get("objectMarking", None)
+        object_label = kwargs.get("objectLabel", None)
+        external_references = kwargs.get("externalReferences", None)
+        revoked = kwargs.get("revoked", None)
+        confidence = kwargs.get("confidence", None)
+        lang = kwargs.get("lang", None)
         created = kwargs.get("created", None)
         modified = kwargs.get("modified", None)
-        created_by_ref = kwargs.get("createdByRef", None)
-        marking_definitions = kwargs.get("markingDefinitions", None)
-        tags = kwargs.get("tags", None)
-        organization_class = kwargs.get("organization_class", None)
+        name = kwargs.get("name", None)
+        description = kwargs.get("description", "")
+        contact_information = kwargs.get("contact_information", None)
+        roles = kwargs.get("roles", None)
+        x_opencti_aliases = kwargs.get("x_opencti_aliases", None)
+        x_opencti_organization_type = kwargs.get("x_opencti_organization_type", None)
+        x_opencti_reliability = kwargs.get("x_opencti_reliability", None)
+        x_opencti_firstname = kwargs.get("x_opencti_firstname", None)
+        x_opencti_lastname = kwargs.get("x_opencti_lastname", None)
+        update = kwargs.get("update", False)
 
         if name is not None and description is not None:
             self.opencti.log("info", "Creating Identity {" + name + "}.")
             input_variables = {
+                "stix_id": stix_id,
+                "createdBy": created_by,
+                "objectMarking": object_marking,
+                "objectLabel": object_label,
+                "externalReferences": external_references,
+                "revoked": revoked,
+                "confidence": confidence,
+                "lang": lang,
+                "created": created,
+                "modified": modified,
                 "name": name,
                 "description": description,
                 "contact_information": contact_information,
-                "alias": alias,
-                "internal_id_key": id,
-                "stix_id_key": stix_id_key,
-                "created": created,
-                "modified": modified,
-                "createdByRef": created_by_ref,
-                "markingDefinitions": marking_definitions,
-                "tags": tags,
+                "roles": roles,
+                "x_opencti_aliases": x_opencti_aliases,
+                "update": update,
             }
             if type == IdentityTypes.ORGANIZATION.value:
                 query = """
                     mutation OrganizationAdd($input: OrganizationAddInput) {
                         organizationAdd(input: $input) {
                             id
-                            stix_id_key
+                            standard_id
                             entity_type
                             parent_types
                         }
                     }
                 """
-                input_variables["organization_class"] = organization_class
+                input_variables[
+                    "x_opencti_organization_type"
+                ] = x_opencti_organization_type
+                input_variables["x_opencti_reliability"] = x_opencti_reliability
                 result_data_field = "organizationAdd"
+            elif type == IdentityTypes.INDIVIDUAL.value:
+                query = """
+                    mutation IndividualAdd($input: IndividualAddInput) {
+                        individualAdd(input: $input) {
+                            id
+                            standard_id
+                            entity_type
+                            parent_types
+                        }
+                    }
+                """
+                input_variables["x_opencti_firstname"] = x_opencti_firstname
+                input_variables["x_opencti_lastname"] = x_opencti_lastname
+                result_data_field = "individualAdd"
             else:
                 query = """
                     mutation IdentityAdd($input: IdentityAddInput) {
                         identityAdd(input: $input) {
                             id
-                            stix_id_key
+                            standard_id
                             entity_type
                             parent_types
                         }
@@ -268,129 +309,18 @@ class Identity:
                 """
                 input_variables["type"] = type
                 result_data_field = "identityAdd"
-            result = self.opencti.query(query, {"input": input_variables,},)
+            result = self.opencti.query(
+                query,
+                {
+                    "input": input_variables,
+                },
+            )
             return self.opencti.process_multiple_fields(
                 result["data"][result_data_field]
             )
         else:
             self.opencti.log("error", "Missing parameters: name and description")
 
-    """
-        Create a  Identity object only if it not exists, update it on request
-
-        :param name: the name of the Identity
-        :return Identity object
-    """
-
-    def create(self, **kwargs):
-        type = kwargs.get("type", None)
-        name = kwargs.get("name", None)
-        description = kwargs.get("description", None)
-        contact_information = kwargs.get("contact_information", None)
-        alias = kwargs.get("alias", None)
-        id = kwargs.get("id", None)
-        stix_id_key = kwargs.get("stix_id_key", None)
-        created = kwargs.get("created", None)
-        modified = kwargs.get("modified", None)
-        created_by_ref = kwargs.get("createdByRef", None)
-        marking_definitions = kwargs.get("markingDefinitions", None)
-        tags = kwargs.get("tags", None)
-        organization_class = kwargs.get("organization_class", None)
-        update = kwargs.get("update", False)
-        custom_attributes = """
-            id
-            entity_type
-            name
-            description 
-            alias
-            ... on Identity {
-                contact_information
-            }
-            ... on Organization {
-                organization_class
-            }
-            createdByRef {
-                node {
-                    id
-                }
-            }            
-        """
-        object_result = self.opencti.stix_domain_entity.get_by_stix_id_or_name(
-            types=[type],
-            stix_id_key=stix_id_key,
-            name=name,
-            customAttributes=custom_attributes,
-        )
-        if object_result is not None:
-            if update or object_result["createdByRefId"] == created_by_ref:
-                # name
-                if object_result["name"] != name:
-                    self.opencti.stix_domain_entity.update_field(
-                        id=object_result["id"], key="name", value=name
-                    )
-                    object_result["name"] = name
-                # description
-                if (
-                    self.opencti.not_empty(description)
-                    and object_result["description"] != description
-                ):
-                    self.opencti.stix_domain_entity.update_field(
-                        id=object_result["id"], key="description", value=description
-                    )
-                    object_result["description"] = description
-                # contact_information
-                if (
-                    self.opencti.not_empty(contact_information)
-                    and object_result["contact_information"] != contact_information
-                ):
-                    self.opencti.stix_domain_entity.update_field(
-                        id=object_result["id"],
-                        key="contact_information",
-                        value=contact_information,
-                    )
-                    object_result["contact_information"] = contact_information
-                # alias
-                if self.opencti.not_empty(alias) and object_result["alias"] != alias:
-                    if "alias" in object_result:
-                        new_aliases = object_result["alias"] + list(
-                            set(alias) - set(object_result["alias"])
-                        )
-                    else:
-                        new_aliases = alias
-                    self.opencti.stix_domain_entity.update_field(
-                        id=object_result["id"], key="alias", value=new_aliases
-                    )
-                    object_result["alias"] = new_aliases
-                # organization_class
-                if (
-                    self.opencti.not_empty(organization_class)
-                    and "organization_class" in object_result
-                    and object_result["organization_class"] != organization_class
-                ):
-                    self.opencti.stix_domain_entity.update_field(
-                        id=object_result["id"],
-                        key="organization_class",
-                        value=organization_class,
-                    )
-                    object_result["organization_class"] = organization_class
-            return object_result
-        else:
-            return self.create_raw(
-                type=type,
-                name=name,
-                description=description,
-                contact_information=contact_information,
-                alias=alias,
-                id=id,
-                stix_id_key=stix_id_key,
-                created=created,
-                modified=modified,
-                createdByRef=created_by_ref,
-                markingDefinitions=marking_definitions,
-                tags=tags,
-                organization_class=organization_class,
-            )
-
     """
         Import an Identity object from a STIX2 object
 
@@ -403,21 +333,34 @@ class Identity:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-            if CustomProperties.IDENTITY_TYPE in stix_object:
-                type = stix_object[CustomProperties.IDENTITY_TYPE].capitalize()
-            else:
+            type = "Organization"
+            if "identity_class" in stix_object:
                 if stix_object["identity_class"] == "individual":
-                    type = "User"
-                elif stix_object["identity_class"] == "organization":
-                    type = "Organization"
-                elif stix_object["identity_class"] == "group":
-                    type = "Organization"
+                    type = "Individual"
                 elif stix_object["identity_class"] == "class":
                     type = "Sector"
-                else:
-                    return None
             return self.create(
                 type=type,
+                stix_id=stix_object["id"],
+                createdBy=extras["created_by_id"]
+                if "created_by_id" in extras
+                else None,
+                objectMarking=extras["object_marking_ids"]
+                if "object_marking_ids" in extras
+                else [],
+                objectLabel=extras["object_label_ids"]
+                if "object_label_ids" in extras
+                else [],
+                externalReferences=extras["external_references_ids"]
+                if "external_references_ids" in extras
+                else [],
+                revoked=stix_object["revoked"] if "revoked" in stix_object else None,
+                confidence=stix_object["confidence"]
+                if "confidence" in stix_object
+                else None,
+                lang=stix_object["lang"] if "lang" in stix_object else None,
+                created=stix_object["created"] if "created" in stix_object else None,
+                modified=stix_object["modified"] if "modified" in stix_object else None,
                 name=stix_object["name"],
                 description=self.opencti.stix2.convert_markdown(
                     stix_object["description"]
@@ -428,23 +371,20 @@ class Identity:
                     stix_object["contact_information"]
                 )
                 if "contact_information" in stix_object
-                else "",
-                alias=self.opencti.stix2.pick_aliases(stix_object),
-                id=stix_object[CustomProperties.ID]
-                if CustomProperties.ID in stix_object
                 else None,
-                stix_id_key=stix_object["id"] if "id" in stix_object else None,
-                created=stix_object["created"] if "created" in stix_object else None,
-                modified=stix_object["modified"] if "modified" in stix_object else None,
-                createdByRef=extras["created_by_ref_id"]
-                if "created_by_ref_id" in extras
+                roles=stix_object["roles"] if "roles" in stix_object else None,
+                x_opencti_aliases=self.opencti.stix2.pick_aliases(stix_object),
+                x_opencti_organization_type=stix_object["x_opencti_organization_type"]
+                if "x_opencti_organization_type" in stix_object
                 else None,
-                markingDefinitions=extras["marking_definitions_ids"]
-                if "marking_definitions_ids" in extras
-                else [],
-                tags=extras["tags_ids"] if "tags_ids" in extras else [],
-                organization_class=stix_object[CustomProperties.ORG_CLASS]
-                if CustomProperties.ORG_CLASS in stix_object
+                x_opencti_reliability=stix_object["x_opencti_reliability"]
+                if "x_opencti_reliability" in stix_object
+                else None,
+                x_opencti_firstname=stix_object["x_opencti_firstname"]
+                if "x_opencti_firstname" in stix_object
+                else None,
+                x_opencti_lastname=stix_object["x_opencti_lastname"]
+                if "x_opencti_lastname" in stix_object
                 else None,
                 update=update,
             )
@@ -452,58 +392,3 @@ class Identity:
             self.opencti.log(
                 "error", "[opencti_identity] Missing parameters: stixObject"
             )
-
-    """
-        Export an Identity object in STIX2
-    
-        :param id: the id of the Identity
-        :return Identity object
-    """
-
-    def to_stix2(self, **kwargs):
-        id = kwargs.get("id", None)
-        mode = kwargs.get("mode", "simple")
-        max_marking_definition_entity = kwargs.get(
-            "max_marking_definition_entity", None
-        )
-        entity = kwargs.get("entity", None)
-        if id is not None and entity is None:
-            entity = self.read(id=id)
-        if entity is not None:
-            if entity["entity_type"] == "user":
-                identity_class = "individual"
-            elif entity["entity_type"] == "sector":
-                identity_class = "class"
-            else:
-                identity_class = "organization"
-            identity = dict()
-            identity["id"] = entity["stix_id_key"]
-            identity["type"] = "identity"
-            identity["spec_version"] = SPEC_VERSION
-            identity["name"] = entity["name"]
-            identity["identity_class"] = identity_class
-            if self.opencti.not_empty(entity["stix_label"]):
-                identity["labels"] = entity["stix_label"]
-            else:
-                identity["labels"] = ["identity"]
-            if self.opencti.not_empty(entity["description"]):
-                identity["description"] = entity["description"]
-            if self.opencti.not_empty(entity["contact_information"]):
-                identity["contact_information"] = entity["contact_information"]
-            identity["created"] = self.opencti.stix2.format_date(entity["created"])
-            identity["modified"] = self.opencti.stix2.format_date(entity["modified"])
-            if self.opencti.not_empty(entity["alias"]):
-                identity["aliases"] = entity["alias"]
-            if (
-                entity["entity_type"] == "organization"
-                and "organization_class" in entity
-                and self.opencti.not_empty(entity["organization_class"])
-            ):
-                identity[CustomProperties.ORG_CLASS] = entity["organization_class"]
-            identity[CustomProperties.IDENTITY_TYPE] = entity["entity_type"]
-            identity[CustomProperties.ID] = entity["id"]
-            return self.opencti.stix2.prepare_export(
-                entity, identity, mode, max_marking_definition_entity
-            )
-        else:
-            self.opencti.log("error", "Missing parameters: id or entity")
